
import asynchat
import asyncore
import socket
import re

routes = {}
routes_to_re = {}
class route:
    def __init__(self,route):
        '''
        route has to be a valid path, including parameters.
        Example:
        "/say/<name>/
        '''
        reg = re.sub("\<.*\>",self.repl,route)

        self.re = re.compile(reg)

        self.route = route


    def matches(self,route):
        '''
        check for match with route
        '''
        return self.re.match(route)

    def repl(self,matcho):
        '''
        Replace function for regexp. Replaces <name> with (?P<name>.*),
        thus making it a valid regexp
        '''
        name = matcho.group()[1:-1]
        return "(?P<{0}>.*)".format(name)

    def __call__(self,func):
        '''
        Function to be able to use the route as a decorator.
        '''
        routes[self] = func
        return func


class kame:
    def __init__(self,path,headers,server):
        self.path,self.headers,self.server = path,headers,server

    def handle(self):
        '''
        calls the function that belongs to the given path and sends
        the return values to the client.
        '''
        groupsdict,function = self.route(self.path)

        self.server.socket.send(str(function(self,**groupsdict)))
        #self.server.socket.close()

    def route(self,path):
        '''
        tries to find a route object for the path. Defaults to the
        error handler if no path could be found.
        '''
        #print "routing %s" % path
        for r in routes.keys():
            ret = r.matches(path)
            if ret:
                retv = ret.groupdict(),routes[r]
                return retv
        
        return self.route("/error/")

@route("/error/")
def error(server):
    return "File not found: %s" % server.path

class kame_request_handler(asynchat.async_chat):

    def __init__(self, sock, addr, sessions, log):
        asynchat.async_chat.__init__(self, sock=sock)
        self.addr = addr
        self.sessions = sessions
        self.ibuffer = []
        self.obuffer = ""
        self.set_terminator("\r\n\r\n")
        self.reading_headers = True
        self.handling = False
        self.cgi_data = None
        self.log = log

    def collect_incoming_data(self, data):
        """Buffer the data"""
        self.ibuffer.append(data)

    def found_terminator(self):
        '''
        called when the terminator (set by set_terminator) is found. 
        Determinates the method (GET / POST / etc), and dispatches the
        request to self.handle_request. If the method is POST it sets
        a new terminator, to read the post data.
        '''
        if self.reading_headers:
            self.reading_headers = False
            self.parse_headers("".join(self.ibuffer))
            self.ibuffer = []
            if self.op.upper() == "POST":
                clen = self.headers.getheader("content-length")
                self.set_terminator(int(clen))
            else:
                self.handling = True
                self.set_terminator(None)
                self.handle_request()
        elif not self.handling:
            self.set_terminator(None) # browsers sometimes over-send
            self.cgi_data = parse(self.headers, "".join(self.ibuffer))
            self.handling = True
            self.ibuffer = []
            self.handle_request()

    def parse_headers(self,headers):
        '''
        Parses the HTTP headers
        '''
        headers = headers.split("\r\n")

        method = headers[0].split(" ")
        headers = headers[1:]

        op = method[0]
        path = method[1]

        headersdict = {}
        for i in headers:
            curhead = i.split(": ")
            headersdict[curhead[0]] = curhead[1]

        self.op = op
        self.path = path
        self.headers = headersdict

    def handle_request(self):
        '''
        dispatches the request to kame
        '''
        self.fw = kame(self.path,self.headers,self)
        self.fw.handle()
        #self.send(self.path)
        self.close()

class KameServer(asyncore.dispatcher):

    def __init__(self, host, port):
        asyncore.dispatcher.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.set_reuse_addr()
        self.bind((host, port))
        self.listen(5)

    def handle_accept(self):
        '''
        Handles a new connection
        '''
        pair = self.accept()
        if pair is None:
            pass
        else:
            sock, addr = pair

            handler = kame_request_handler(sock,addr,{},"")




def run(server='localhost', port=8080):
    '''
    Runs the Kame server. Port defaults to 8080
    '''
    serv = KameServer(server, port)
    print "kame running on {server}:{port}".format(server=server,port=port)
    asyncore.loop()
