import json
import os

CONFIG_PATH = os.path.expanduser('~/.vomun/config.json')

## Setup global variables
globals = {
    'running': True,
    'anon+': {
        'VERSION': 'v0.2.1',
        'BUILD': 6,
    }
}

globals['anon+']['banner'] = '''
======================
= Anon+ %s
= Build: %s
======================
'''

## Define classes
class Configuration(dict):
    '''A class to hold the contents of configuration files.'''
    path = CONFIG_PATH
    def load(self):
        '''Load the configuration file from the location set in self.path.
        The loaded data is parsed for json contents.
        '''
        try:
            with open(self.path, 'r') as config_file:
                values = json.loads(config_file.read())
        except IOError:
            self['vomundir'] = os.path.expanduser('~/.vomun/')
            self['nodekey'] = ''
            self.save()
        else:
            for value in values:
                self[value] = values[value]

    def save(self):
        '''Save the configuration enteries in a json format to the location set
        in self.path.
        '''
        with open(self.path, 'w') as config_file:
            config_file.write(json.dumps(self, indent=2))
            
class Registrar(dict):
    '''Manages dynamic imports relative to a set start_path.'''
    def __init__(self, start_path = ''):
        self.start_path = start_path
        
    def __getitem__(self, item):
        if item not in self:
            self[item] = getattr(__import__(self.start_path + item), item)
            return self[item]
        else:
            return super(Registrar, self).__getitem__(item)

## Load the configuration file
config = Configuration()
config.load()
